----------------------------------------------------------------------------------

-- Module Name:    cartTop - Behavioral 

----------------------------------------------------------------------------------

-- package SIGDECBIT is
-- signal VCC_SIGNAL    : bit := '1';
-- signal GROUND_SIGNAL : bit := '0';
-- end SIGDECBIT;

-- USE SIGDECBIT.ALL;

library IEEE;
use IEEE.STD_LOGIC_1164.ALL;




-- Uncomment the following library declaration if using
-- arithmetic functions with Signed or Unsigned values
--use IEEE.NUMERIC_STD.ALL;

-- Uncomment the following library declaration if instantiating
-- any Xilinx primitives in this code.
--library UNISIM;
--use UNISIM.VComponents.all;

entity cartTop is
    Port ( 

				A0    : in    std_logic; 
				A1    : in    std_logic; 
				A2    : in    std_logic; 
				A3    : in    std_logic; 
				A4    : in    std_logic; 
				A5    : in    std_logic; 
				A6    : in    std_logic; 
				A7    : in    std_logic; 
				
				A13    : in    std_logic; 
				A14    : in    std_logic; 
				A15    : in    std_logic; 
				
				D0    : inout    std_logic; 
				D1    : inout    std_logic; 
				D2    : inout    std_logic; 
				D3    : inout    std_logic; 
				D4    : inout    std_logic; 
				D5    : inout    std_logic; 
				D6    : inout    std_logic; 
				D7    : inout    std_logic; 
				
				FI2   : in    std_logic; 
				
				IO1   : in    std_logic; 
				IO2   : in    std_logic; 
				RESET : in    std_logic; 
				RW    : in    std_logic; 
				
				--ROM_LOW   : in std_logic;
				
				
				O_A13   : out   std_logic; 
				O_A14   : out   std_logic; 
				O_A15   : out   std_logic; 
				O_A16   : out   std_logic; 
				O_A17   : out   std_logic; 
				O_A18   : out   std_logic;
				
				EXROM : out   std_logic; 
				O_CE   : inout std_logic;
				O_OE   : out std_logic;
				BUFEX   : out std_logic;
				
				
				NMI_OUT   : out   std_logic; 
				MEM_CS    : out   std_logic; 
				MEM_WE    : out   std_logic; 
				GAME      : out   std_logic 
				
				
				
				
		  );

		  
       
end cartTop;
 
architecture Behavioral of cartTop is


component IO_gate_trigger
    Port ( i1 : in  STD_LOGIC;
           i2 : in  STD_LOGIC;
           i3 : in  STD_LOGIC;
           trig : out  STD_LOGIC);
end component;

component syncgate 
    Port ( inp1  : in  STD_LOGIC;
           inp2  : in  STD_LOGIC;
			  inp3  : in  STD_LOGIC;
			  inp4  : in  STD_LOGIC;
			  inp5  : in  STD_LOGIC;
			  inp6  : in  STD_LOGIC;

           otp1 : out  STD_LOGIC
			  );
end component;

component decodezero 
    Port ( i1 : in  STD_LOGIC;
           i2 : in  STD_LOGIC;
           zerotrig : out  STD_LOGIC);
end component;


	
	component Reg8B
	    Port ( Res : in  STD_LOGIC;
           Clk : in  STD_LOGIC;
			  ACK : in  STD_LOGIC;
           D : in  STD_LOGIC_VECTOR (7 downto 0);
           Q : out  STD_LOGIC_VECTOR (7 downto 0));
	end component;


COMPONENT TRISTATEBUFFER
    Port ( 
	  X  :  in  STD_LOGIC_VECTOR (7 downto 0);
			  
           nOE : in  STD_LOGIC;
			  
			  Y  : out  STD_LOGIC_VECTOR (7 downto 0)
			  );
END COMPONENT;


COMPONENT read_ACK 
    Port ( i1 : in  STD_LOGIC;
           i2 : in  STD_LOGIC;
           i3 : in  STD_LOGIC;
           i4 : in  STD_LOGIC;
			  i5 : in  STD_LOGIC;
           o : out  STD_LOGIC);
END COMPONENT;

component chipselectgate 
    Port ( i1 : in  STD_LOGIC;
           i2 : in  STD_LOGIC;
           out1 : out  STD_LOGIC);
end component;


component ACTIVATE_RAM_GATE
    Port ( I1 : in  STD_LOGIC;
           I2 : in  STD_LOGIC;
           I3 : in  STD_LOGIC;
           O : out  STD_LOGIC);
END component;


component MEM_CHIP_SELECT 
    Port ( I1 : in  STD_LOGIC;
           I2 : in  STD_LOGIC;
           O : out  STD_LOGIC);
END component;


component RAM_WRITE_ENABLE
    Port ( I1 : in  STD_LOGIC;
           I2 : in  STD_LOGIC;
           O : out  STD_LOGIC);
end component;



--------------------------------------------------------------
--------------------------------------------------------------
	signal VCC_SIGNAL    : std_logic := '1';
   -- signal GROUND_SIGNAL : std_logic := '0';


	signal SIX_SIGNAL : STD_LOGIC;
	signal ZERO_SIGNAL : STD_LOGIC;
	signal IOTRIG_SIGNAL : STD_LOGIC;
	
	signal ACK_SIGNAL : STD_LOGIC;
	signal ACK_READ   : STD_LOGIC;
	
	signal A13_SIGNAL : STD_LOGIC;
	signal A14_SIGNAL : STD_LOGIC;
	signal A15_SIGNAL : STD_LOGIC;
	signal A16_SIGNAL : STD_LOGIC;
	signal A17_SIGNAL : STD_LOGIC;
	signal A18_SIGNAL : STD_LOGIC;
	signal EXROM_SIGNAL : STD_LOGIC;
	signal CHIPSELECT_SIGNAL : STD_LOGIC := '1';
	signal BUFEX_SIGNAL : STD_LOGIC;
	
	
	signal RAM_ACTIVATE_SIGNAL : STD_LOGIC;
	-----------------------------------------signal RAM_READ_WRITE_SIGNAL : STD_LOGIC;
	
	
begin


		-- if IO1 RW CHIPSELECT_SIGNAL = 0 then trig = 0 else 1
		IOgate1 : IO_gate_trigger
		port map
		( 
			i1  => IO1,
			i2  => RW,
			i3  => CHIPSELECT_SIGNAL,
			
			trig => IOTRIG_SIGNAL
		);

		-- if a2-a6 = 0 then trig = 0 else 1
		gate1 : syncgate
		port map
		( 
			
			inp1  => A2,
			inp2  => A3,
			inp3  => A4,
			inp4  => A5,
			inp5  => A6,
			inp6  => A7,

			otp1 => SIX_SIGNAL
		);
		
		-- if a0 a1 = 0 then zerotrig = 0 else 1
		gate2 : decodezero
		port map
		( 
			i1  => A0,
			i2  => A1,
			
			zerotrig => ZERO_SIGNAL
		);
		
		
		-- if 3signals = 0 then ACK_SIGNAL = 0 = write allowed
		write_strobe : IO_gate_trigger
		port map
		( 
			i1  => ZERO_SIGNAL,
			i2  => SIX_SIGNAL,
			i3  => IOTRIG_SIGNAL,
			
			trig => ACK_SIGNAL
		);
		
		WRITE_REGISTER : Reg8B
		port map
		(
			Clk 		=> FI2,
			ACK 		=> ACK_SIGNAL,
			Res 		=> RESET,
		   D(0) 		=> D0,
		   D(1) 		=> D1,
		   D(2) 		=> D2,
		   D(3) 		=> D3,
		   D(4) 		=> D4,
		   D(5) 		=> D5,
		   D(6) 		=> D6,
		   D(7) 		=> D7,
		   Q(0) 		=> A18_SIGNAL,
		   Q(1) 		=> EXROM_SIGNAL,
		   Q(2) 		=> CHIPSELECT_SIGNAL,
		   Q(3) 		=> A13_SIGNAL,
		   Q(4) 		=> A14_SIGNAL,
		   Q(5) 		=> A15_SIGNAL,
		   Q(6) 		=> A16_SIGNAL,
		   Q(7) 		=> A17_SIGNAL	
		);
		
		
	READBUF : TRISTATEBUFFER
		port map
		(

			nOE 		=> ACK_READ,

		   Y(0) 		=> D0,
		   Y(1) 		=> D1,
		   Y(2) 		=> D2,
		   Y(3) 		=> D3,
		   Y(4) 		=> D4,
		   Y(5) 		=> D5,
		   Y(6) 		=> D6,
		   Y(7) 		=> D7,
		   X(0) 		=> A18_SIGNAL,
		   X(1) 		=> EXROM_SIGNAL,
		   X(2) 		=> CHIPSELECT_SIGNAL,
		   X(3) 		=> A13_SIGNAL,
		   X(4) 		=> A14_SIGNAL,
		   X(5) 		=> A15_SIGNAL,
		   X(6) 		=> A16_SIGNAL,
		   X(7) 		=> A17_SIGNAL	
		);	
		
		-- if RW = 1 and other 4 signals = 0 then ACK_READ signal = 0 (read allowed)
			gate_READ_IO1 : read_ACK 
		port map
		( 
			i1  => RW, -- must be exactly here !
			i2  => IO1,
			i3  => CHIPSELECT_SIGNAL,
			i4  => SIX_SIGNAL,
			i5  => ZERO_SIGNAL,
			
			o => ACK_READ
		);	
		
		gate3 : chipselectgate
		port map
		( 
			i1  => RAM_ACTIVATE_SIGNAL,
			i2  => IO2,
			
			out1 => BUFEX_SIGNAL
		);
		
		TEST_RAM_INVOKE : ACTIVATE_RAM_GATE
		port map
		( 
			i1  => A15, -- must be exactly here !
			i2  => A14,
			i3  => A13,

			
			o => RAM_ACTIVATE_SIGNAL
		);	
		
		INVOKE_RAM : MEM_CHIP_SELECT
		port map
		( 
			i1  => CHIPSELECT_SIGNAL, 
			i2  => BUFEX_SIGNAL,
			
			o => MEM_CS
		);
		
		
		
		RAM_Read_Write : RAM_WRITE_ENABLE
		port map
		( 
			i1  => FI2, -- must be exactly here ! 
			i2  => RW,
			
			o => MEM_WE
		);		
		
		
		
					----------------------------------------
					--             H.E.R.E !!!            --
					-- our entity work connect to signals --
					----------------------------------------
		



		
			O_A13 <= A13_SIGNAL ;
			O_A14 <= A14_SIGNAL ;
			O_A15 <= A15_SIGNAL ;
			O_A16 <= A16_SIGNAL ;
			O_A17 <= A17_SIGNAL ;
			O_A18 <= A13_SIGNAL ;

			EXROM  <= EXROM_SIGNAL ;
			BUFEX  <= BUFEX_SIGNAL;
			
			
			O_CE  	 <= VCC_SIGNAL;
			O_OE  	 <= VCC_SIGNAL;
			GAME      <= VCC_SIGNAL;			
			NMI_OUT   <= VCC_SIGNAL;

			-------------------------MEM_WE    <= RAM_READ_WRITE_SIGNAL;

				
			
end Behavioral;

